package extmedias

import (
	"database/sql"
	"errors"
	"fmt"
	"net/http"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/auth"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/movies"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/shows"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/users"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/web"

	"github.com/Sirupsen/logrus"
	polochon "github.com/odwrtw/polochon/lib"
	eztvExplorer "github.com/odwrtw/polochon/modules/eztv"
	traktExplorer "github.com/odwrtw/polochon/modules/trakttv"
	ytsExplorer "github.com/odwrtw/polochon/modules/yts"
)

// GetMediaIDs will get some media IDs
func GetMediaIDs(env *web.Env, mediaType string, source string, category string, force bool) ([]string, error) {
	log := env.Log.WithFields(logrus.Fields{
		"mediaType": mediaType,
		"source":    source,
		"category":  category,
		"function":  "extmedias.GetMediaIds",
	})

	var err error
	media, err := Get(env.Database, mediaType, source, category)
	switch err {
	case nil:
		log.Debugf("%s medias found in database", mediaType)
		if !force {
			log.Debug("returning medias from db")
			return media.IDs, nil
		}
	case sql.ErrNoRows:
		log.Debugf("%s medias not found in database", mediaType)
	default:
		// Unexpected error
		return nil, err
	}

	explorer, err := NewExplorer(env, source)
	if err != nil {
		return nil, err
	}

	var ids []string
	if mediaType == "movie" {
		medias, err := explorer.GetMovieList(polochon.ExploreByRate, log)
		if err != nil {
			return nil, err
		}
		for _, media := range medias {
			ids = append(ids, media.ImdbID)
		}
	} else {
		medias, err := explorer.GetShowList(polochon.ExploreByRate, log)
		if err != nil {
			return nil, err
		}
		for i, media := range medias {
			if i > 5 {
				break
			}
			ids = append(ids, media.ImdbID)
		}
	}

	log.Debugf("got %d medias from %s", len(ids), source)

	media = &Media{
		Type:     mediaType,
		Source:   source,
		Category: category,
		IDs:      ids,
	}

	err = media.Upsert(env.Database)
	if err != nil {
		return nil, err
	}

	log.Debug("medias updated in database")

	return ids, nil
}

// GetMovies get some movies
func GetMovies(env *web.Env, user *users.User, source string, category string, force bool) ([]*movies.Movie, error) {
	movieIds, err := GetMediaIDs(env, "movie", source, category, force)
	if err != nil {
		return nil, err
	}

	// Get the URLs from polochon, we don't really care if it fails for now
	var urls map[string]string
	urls, err = movies.GetPolochonMoviesURLs(user)
	if err != nil {
		env.Log.Errorf("error while getting polochon movies url: %s", err)
	}

	movieList := []*movies.Movie{}
	for _, id := range movieIds {
		movie := movies.New(id)
		err := movie.GetDetails(env, force)
		if err != nil {
			env.Log.Errorf("error while getting movie details : %s", err)
			continue
		}
		err = movie.GetTorrents(env, force)
		if err != nil {
			env.Log.Errorf("error while getting movie torrents : %s", err)
			continue
		}

		if urls != nil {
			if url, ok := urls[id]; ok {
				movie.PolochonURL = url
			}
		}

		movieList = append(movieList, movie)
	}
	return movieList, nil
}

// GetShows get some shows
func GetShows(env *web.Env, source string, category string, force bool) ([]*shows.Show, error) {
	showIds, err := GetMediaIDs(env, "show", source, category, force)
	if err != nil {
		return nil, err
	}

	showList := []*shows.Show{}
	for _, id := range showIds {
		show := shows.New(id)
		err := show.GetDetails(env, force)
		if err != nil {
			env.Log.Errorf("error while getting show details : %s", err)
			continue
		}
		err = show.GetTorrents(env, force)
		if err != nil {
			env.Log.Errorf("error while getting show torrents : %s", err)
			continue
		}
		showList = append(showList, show)
	}
	return showList, nil
}

// Explore will explore some movies
func Explore(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	err := r.ParseForm()
	if err != nil {
		return err
	}

	source := r.FormValue("source")
	// Default source
	if source == "" {
		source = "yts"
	}

	category := r.FormValue("category")
	// Default category
	if category == "" {
		category = "popular"
	}

	v := auth.GetCurrentUser(r, env.Log)
	user, ok := v.(*users.User)
	if !ok {
		return env.RenderError(w, errors.New("invalid user"))
	}

	// Get the medias without trying to refresh them
	movies, err := GetMovies(env, user, source, category, false)
	if err != nil {
		return env.RenderError(w, err)
	}

	return env.RenderJSON(w, movies)
}

// ExploreShows will explore some shows
func ExploreShows(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	err := r.ParseForm()
	if err != nil {
		return err
	}

	source := r.FormValue("source")
	// Default source
	if source == "" {
		source = "eztv"
	}

	category := r.FormValue("category")
	// Default category
	if category == "" {
		category = "popular"
	}

	// Get the medias without trying to refresh them
	shows, err := GetShows(env, source, category, false)
	if err != nil {
		return env.RenderError(w, err)
	}

	return env.RenderJSON(w, shows)
}

// MediaSources represents the implemented media sources
var MediaSources = []string{
	"trakttv",
	"yts",
}

// ShowMediaSources represents the implemented media sources for shows
var ShowMediaSources = []string{
	"eztv",
}

// Refresh will refresh the movie list
func Refresh(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	env.Log.Debugf("refreshing infos ...")
	source := r.FormValue("source")
	if source == "" {
		source = "yts"
	}

	category := r.FormValue("category")
	if category == "" {
		category = "popular"
	}

	v := auth.GetCurrentUser(r, env.Log)
	user, ok := v.(*users.User)
	if !ok {
		return fmt.Errorf("invalid user type")
	}

	// We'll refresh the medias for each sources
	for _, source := range MediaSources {
		env.Log.Debugf("refreshing %s", source)
		// GetMedias and refresh them
		_, err := GetMovies(env, user, source, category, true)
		if err != nil {
			return env.RenderError(w, err)
		}
	}

	return env.RenderJSON(w, map[string]string{"message": "Refresh is done"})
}

// RefreshShows will refresh the movie list
func RefreshShows(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	env.Log.Debugf("refreshing shows ...")
	source := r.FormValue("source")
	if source == "" {
		source = "eztv"
	}

	category := r.FormValue("category")
	if category == "" {
		category = "popular"
	}

	// We'll refresh the medias for each sources
	for _, source := range ShowMediaSources {
		env.Log.Debugf("refreshing %s", source)
		// GetMedias and refresh them
		_, err := GetShows(env, source, category, true)
		if err != nil {
			return env.RenderError(w, err)
		}
	}

	return env.RenderJSON(w, map[string]string{"message": "Refresh is done"})
}

// NewExplorer returns a polochon.Explorer
func NewExplorer(env *web.Env, source string) (polochon.Explorer, error) {
	switch source {
	case "trakttv":
		return traktExplorer.NewExplorer(&traktExplorer.Params{
			ClientID: env.Config.TraktTVClientID,
		})
	case "yts":
		return ytsExplorer.NewExplorer()
	case "eztv":
		return eztvExplorer.NewExplorer()
	default:
		return nil, fmt.Errorf("unknown explorer")
	}
}
