package torrents

import (
	"database/sql"
	"fmt"
	"time"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/sqly"

	"github.com/jmoiron/sqlx"
	polochon "github.com/odwrtw/polochon/lib"
)

const (
	upsertEpisodeTorrentQuery = `
	INSERT INTO episode_torrents (imdb_id, url, source, quality, upload_user,
	season, episode, seeders, leechers)
	VALUES (:imdb_id, :url, :source, :quality, :upload_user, :season, :episode,
	:seeders, :leechers)
	ON CONFLICT (imdb_id, season, episode, quality, source)
	DO UPDATE SET imdb_id=:imdb_id, url=:url, source=:source, quality=:quality,
	upload_user=:upload_user, season=:season, episode=:episode,
	seeders=:seeders, leechers=:leechers
	RETURNING id;`

	getEpisodeTorrentQuery = `
	SELECT *
	FROM episode_torrents WHERE imdb_id=$1 AND season=$2 AND episode=$3;`

	getEpisodeTorrentQueryByID = `
	SELECT *
	FROM episode_torrents WHERE id=$1;`

	deleteEpisodeTorrentQuery = `DELETE FROM movie_torrents WHERE id=$1;`
)

// EpisodeTorrent represents an episode torrent
type EpisodeTorrent struct {
	sqly.BaseModel
	polochon.Torrent
	ShowImdbID string `json:"show_imdb_id"`
	Season     int    `json:"season"`
	Episode    int    `json:"episode"`
}

// EpisodeTorrentDB represents the EpisodeTorrent in the DB
type EpisodeTorrentDB struct {
	ID         string    `db:"id"`
	ImdbID     string    `db:"imdb_id"`
	URL        string    `db:"url"`
	Source     string    `db:"source"`
	Quality    string    `db:"quality"`
	UploadUser string    `db:"upload_user"`
	Season     int       `db:"season"`
	Episode    int       `db:"episode"`
	Seeders    int       `db:"seeders"`
	Leechers   int       `db:"leechers"`
	Created    time.Time `db:"created_at"`
	Updated    time.Time `db:"updated_at"`
}

// NewEpisodeFromPolochon returns a new EpisodeTorrent from a polochon.ShowEpisode
func NewEpisodeFromPolochon(se polochon.ShowEpisode, poTo polochon.Torrent) *EpisodeTorrent {
	return &EpisodeTorrent{
		ShowImdbID: se.ShowImdbID,
		Season:     se.Season,
		Episode:    se.Episode,
		Torrent:    poTo,
	}
}

// NewEpisode returns a new EpisodeTorrent
func NewEpisode() *EpisodeTorrent {
	return &EpisodeTorrent{}
}

// GetEpisodeTorrents returns show details in database from id or imdbid or an error
func GetEpisodeTorrents(db *sqlx.DB, imdbID string, season, episode int) ([]*EpisodeTorrent, error) {
	var torrentsDB = []*EpisodeTorrentDB{}
	err := db.Select(&torrentsDB, getEpisodeTorrentQuery, imdbID, season, episode)
	if err != nil {
		return nil, err
	}

	if len(torrentsDB) == 0 {
		return nil, sql.ErrNoRows
	}

	var torrents []*EpisodeTorrent
	for _, torrentDB := range torrentsDB {
		episode := NewEpisode()
		episode.FillFromDB(torrentDB)
		torrents = append(torrents, episode)
	}

	return torrents, nil
}

// Delete episode from database
func (e *EpisodeTorrent) Delete(db *sqlx.DB) error {
	r, err := db.Exec(deleteEpisodeTorrentQuery, e.ID)
	if err != nil {
		return err
	}
	count, _ := r.RowsAffected()
	if count != 1 {
		return fmt.Errorf("Unexpected number of row deleted: %d", count)
	}
	return nil
}

// FillFromDB will fill a MovieTorrent from a MovieTorrentDB extracted from the DB
func (e *EpisodeTorrent) FillFromDB(eDB *EpisodeTorrentDB) {
	q, _ := polochon.StringToQuality(eDB.Quality)
	e.ShowImdbID = eDB.ImdbID
	e.Created = eDB.Created
	e.Updated = eDB.Updated
	e.ID = eDB.ID
	e.URL = eDB.URL
	e.Source = eDB.Source
	e.Quality = *q
	e.UploadUser = eDB.UploadUser
	e.Seeders = eDB.Seeders
	e.Leechers = eDB.Leechers
}

// Upsert an episode torrent in the database
func (e *EpisodeTorrent) Upsert(db *sqlx.DB) error {
	eDB := NewEpisodeTorrentDB(e)
	var id string
	r, err := db.NamedQuery(upsertEpisodeTorrentQuery, eDB)
	if err != nil {
		return err
	}
	for r.Next() {
		r.Scan(&id)
	}
	e.ID = id

	return nil
}

// NewEpisodeTorrentDB returns an EpisodeTorrent ready to be put in DB from a
// EspisodeTorrent
func NewEpisodeTorrentDB(e *EpisodeTorrent) EpisodeTorrentDB {
	return EpisodeTorrentDB{
		ID:         e.ID,
		ImdbID:     e.ShowImdbID,
		Season:     e.Season,
		Episode:    e.Episode,
		URL:        e.URL,
		Source:     e.Source,
		Quality:    string(e.Quality),
		UploadUser: e.UploadUser,
		Seeders:    e.Seeders,
		Leechers:   e.Leechers,
		Created:    e.Created,
		Updated:    e.Updated,
	}
}
